@@ -2,6 +2,7 @@ import re
 from dataclasses import fields, replace
 from functools import wraps
 from typing import Callable, List, Tuple
+from graphql import GraphQLInterfaceType, GraphQLObjectType, GraphQLUnionType
 
 from graphql.error import GraphQLError
 from graphql.execution import ExecutionResult
@@ -22,8 +23,8 @@ from .misc import (
     MISSING_LIMITS,
     ComplexityLimitReached,
     DepthLimitReached,
-    Limits,
     SelectionsLimitReached,
+    Limits,
 )
 
 
@@ -85,7 +86,7 @@ def check_resource_usage(
     node: Node,
     validation_context: ValidationContext,
     limits: Limits,
-    on_error: Callable[[str, Node], None],
+    on_error: Callable[[GraphQLError], None],
     auto_snakecase=False,
     get_limits_for_field=limits_for_field,
     level_depth=0,
@@ -120,11 +121,12 @@ def check_resource_usage(
         if isinstance(field, FragmentSpreadNode):
             field = validation_context.getFragment(field.name.value)
 
-        # union
-        if hasattr(field, "types"):
+        if isinstance(field, (GraphQLUnionType, GraphQLInterfaceType)):
             merged_limits = limits
             local_selections = 0
-            for field_type in field.types:
+            for field_type in validation_context.schema.get_possible_types(
+                field
+            ):
                 (
                     new_depth,
                     new_depth_complexity,
@@ -170,9 +172,11 @@ def check_resource_usage(
                 schema_field = getattr(schema, fieldname)
             except AttributeError:
                 schema_field = schema
-                if hasattr(schema_field, "fields"):
-                    schema_field = follow_of_type(
-                        schema_field.fields[field.name.value]
+
+                # handle Fragment Spread nodes
+                if isinstance(schema_field, FragmentSpreadNode):
+                    schema_field = validation_context.getFragment(
+                        schema_field.name.value
                     )
             merged_limits, sub_limits = get_limits_for_field(
                 schema_field,
@@ -185,7 +189,10 @@ def check_resource_usage(
                 allow_reset = False
             else:
                 _seen_limits.add(id(sub_limits))
-            if hasattr(schema_field, "types"):
+            if isinstance(
+                schema_field,
+                (GraphQLUnionType, GraphQLInterfaceType, GraphQLObjectType),
+            ):
                 sub_field_type = schema_field
             else:
                 sub_field_type = follow_of_type(schema_field.type)
